﻿using System;
using System.ComponentModel;
using System.Threading;

namespace ServiceBusAppDemo.Common
{
    public abstract class MessageReceiver<T> : IMessageReceiver<T> 
        where T : class, new()
    {
        private readonly ServiceBus _serviceBus;
        private BackgroundWorker _worker;

        protected MessageReceiver(ILogger logger)
        {
            Interval = TimeSpan.FromSeconds(20);
            _serviceBus = new ServiceBus(logger);
        }

        public void Start()
        {
            _worker = new BackgroundWorker();
            _worker.DoWork += DoWork;
            _worker.RunWorkerAsync();

        }

        public void Stop()
        {
            _worker.CancelAsync();
            _serviceBus.Dispose();
        }

        private void DoWork(object sender, DoWorkEventArgs e)
        {
            while (!_worker.CancellationPending)
            {
                bool hasMessage = _serviceBus.ReceiveMessage<T>(Software, ProcessMessage);
                while (hasMessage && ProcessAllPendingMessages)
                {
                    Thread.Sleep(TimeSpan.FromMilliseconds(10));
                    hasMessage = _serviceBus.ReceiveMessage<T>(Software, ProcessMessage);
                }
                Thread.Sleep(Interval);
            }
        }


        public abstract bool ProcessMessage(Message<T> message);

        public abstract Software Software { get; }

        protected TimeSpan Interval { get; set; }

        protected bool ProcessAllPendingMessages { get; set; }
    }
}
